<?php

namespace Litecms\Contact\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litecms\Contact\Forms\Landlord as LandlordForm;
use Litecms\Contact\Http\Requests\LandlordResourceRequest;
use Litecms\Contact\Http\Resources\LandlordResource;
use Litecms\Contact\Http\Resources\LandlordsCollection;
use Litecms\Contact\Models\Landlord;
use Litecms\Contact\Actions\LandlordAction;
use Litecms\Contact\Actions\LandlordActions;


/**
 * Resource controller class for landlord.
 */
class LandlordResourceController extends BaseController
{

    /**
     * Initialize landlord resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = LandlordForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('litecms.contact.modules'), 'contact', guard_url('contact'));
            return $next($request);
        });
    }

    /**
     * Display a list of landlord.
     *
     * @return Response
     */
    public function index(LandlordResourceRequest $request)
    {
        $request = $request->all();
        $page = LandlordActions::run('paginate', $request);

        $data = new LandlordsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('contact::landlord.names'))
            ->view('contact::landlord.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display landlord.
     *
     * @param Request $request
     * @param Model   $landlord
     *
     * @return Response
     */
    public function show(LandlordResourceRequest $request, Landlord $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new LandlordResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('contact::landlord.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('contact::landlord.show')
            ->output();
    }

    /**
     * Show the form for creating a new landlord.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(LandlordResourceRequest $request, Landlord $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new LandlordResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('contact::landlord.name'))
            ->view('contact::landlord.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new landlord.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(LandlordResourceRequest $request, Landlord $model)
    {
        try {
            $request = $request->all();
            $model = LandlordAction::run('store', $model, $request);
            $data = new LandlordResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('contact::landlord.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('contact/landlord/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/contact/landlord'))
                ->redirect();
        }

    }

    /**
     * Show landlord for editing.
     *
     * @param Request $request
     * @param Model   $landlord
     *
     * @return Response
     */
    public function edit(LandlordResourceRequest $request, Landlord $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new LandlordResource($model);
        // return view('contact::landlord.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('contact::landlord.name'))
            ->view('contact::landlord.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the landlord.
     *
     * @param Request $request
     * @param Model   $landlord
     *
     * @return Response
     */
    public function update(LandlordResourceRequest $request, Landlord $model)
    {
        try {
            $request = $request->all();
            $model = LandlordAction::run('update', $model, $request);
            $data = new LandlordResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('contact::landlord.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('contact/landlord/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('contact/landlord/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the landlord.
     *
     * @param Model   $landlord
     *
     * @return Response
     */
    public function destroy(LandlordResourceRequest $request, Landlord $model)
    {
        try {

            $request = $request->all();
            $model = LandlordAction::run('destroy', $model, $request);
            $data = new LandlordResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('contact::landlord.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('contact/landlord/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('contact/landlord/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
