<?php

namespace Litecms\Contact\Http\Controllers;

use Exception;
use Litecms\Contact\Actions\LandlordWorkflow;
use Litecms\Contact\Http\Requests\LandlordWorkflowRequest;
use Litecms\Contact\Http\Resources\LandlordResource;
use Litecms\Contact\Models\Landlord;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class LandlordWorkflowController extends BaseController
{
    /**
     * Action controller function for landlord.
     *
     * @param Model $landlord
     * @param action next action for the landlord.
     *
     * @return Response
     */
    public function __invoke(LandlordWorkflowRequest $request, Landlord $landlord, $trasition)
    {
        try {
            $request = $request->all();
            $landlord = LandlordWorkflow::run($trasition, $landlord, $request);
            $data = new LandlordResource($landlord);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('contact::landlord.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('contact/landlord/' . $landlord->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('contact/landlord/' . $landlord->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the landlord and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $landlord
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $landlord, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $landlord = Landlord::findBySignedId($landlord);
        $transitions = $this->transitions($landlord, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('contact::landlord.name') . ' ' . trans('Approvals'))
            ->view('contact::public.landlord.workflow')
            ->layout('mobile')
            ->data(compact('landlord', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the landlord.
     *
     * @param PublicRequest $request
     * @param string $landlord
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $landlord, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $landlord = Landlord::findBySignedId($landlord);
        $user = user();
        $landlord = LandlordWorkflow::run($transition, $landlord, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($landlord, $user)
    {
        $transitions = [];
        foreach ($landlord->workflow()->transitions($landlord) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.landlord.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'landlord' => $landlord->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $landlord->workflow()->form($value);
            $array['label'] = trans('contact::landlord.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
