<?php

namespace Litecms\Contact\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litecms\Contact\Forms\Tenant as TenantForm;
use Litecms\Contact\Http\Requests\TenantResourceRequest;
use Litecms\Contact\Http\Resources\TenantResource;
use Litecms\Contact\Http\Resources\TenantsCollection;
use Litecms\Contact\Models\Tenant;
use Litecms\Contact\Actions\TenantAction;
use Litecms\Contact\Actions\TenantActions;


/**
 * Resource controller class for tenant.
 */
class TenantResourceController extends BaseController
{

    /**
     * Initialize tenant resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = TenantForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('litecms.contact.modules'), 'contact', guard_url('contact'));
            return $next($request);
        });
    }

    /**
     * Display a list of tenant.
     *
     * @return Response
     */
    public function index(TenantResourceRequest $request)
    {
        $request = $request->all();
        $page = TenantActions::run('paginate', $request);

        $data = new TenantsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('contact::tenant.names'))
            ->view('contact::tenant.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display tenant.
     *
     * @param Request $request
     * @param Model   $tenant
     *
     * @return Response
     */
    public function show(TenantResourceRequest $request, Tenant $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TenantResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('contact::tenant.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('contact::tenant.show')
            ->output();
    }

    /**
     * Show the form for creating a new tenant.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(TenantResourceRequest $request, Tenant $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TenantResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('contact::tenant.name'))
            ->view('contact::tenant.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new tenant.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TenantResourceRequest $request, Tenant $model)
    {
        try {
            $request = $request->all();
            $model = TenantAction::run('store', $model, $request);
            $data = new TenantResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('contact::tenant.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('contact/tenant/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/contact/tenant'))
                ->redirect();
        }

    }

    /**
     * Show tenant for editing.
     *
     * @param Request $request
     * @param Model   $tenant
     *
     * @return Response
     */
    public function edit(TenantResourceRequest $request, Tenant $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TenantResource($model);
        // return view('contact::tenant.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('contact::tenant.name'))
            ->view('contact::tenant.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the tenant.
     *
     * @param Request $request
     * @param Model   $tenant
     *
     * @return Response
     */
    public function update(TenantResourceRequest $request, Tenant $model)
    {
        try {
            $request = $request->all();
            $model = TenantAction::run('update', $model, $request);
            $data = new TenantResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('contact::tenant.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('contact/tenant/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('contact/tenant/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the tenant.
     *
     * @param Model   $tenant
     *
     * @return Response
     */
    public function destroy(TenantResourceRequest $request, Tenant $model)
    {
        try {

            $request = $request->all();
            $model = TenantAction::run('destroy', $model, $request);
            $data = new TenantResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('contact::tenant.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('contact/tenant/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('contact/tenant/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
