<?php

namespace Litecms\Contact\Http\Controllers;

use Exception;
use Litecms\Contact\Actions\TenantWorkflow;
use Litecms\Contact\Http\Requests\TenantWorkflowRequest;
use Litecms\Contact\Http\Resources\TenantResource;
use Litecms\Contact\Models\Tenant;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class TenantWorkflowController extends BaseController
{
    /**
     * Action controller function for tenant.
     *
     * @param Model $tenant
     * @param action next action for the tenant.
     *
     * @return Response
     */
    public function __invoke(TenantWorkflowRequest $request, Tenant $tenant, $trasition)
    {
        try {
            $request = $request->all();
            $tenant = TenantWorkflow::run($trasition, $tenant, $request);
            $data = new TenantResource($tenant);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('contact::tenant.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('contact/tenant/' . $tenant->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('contact/tenant/' . $tenant->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the tenant and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $tenant
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $tenant, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $tenant = Tenant::findBySignedId($tenant);
        $transitions = $this->transitions($tenant, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('contact::tenant.name') . ' ' . trans('Approvals'))
            ->view('contact::public.tenant.workflow')
            ->layout('mobile')
            ->data(compact('tenant', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the tenant.
     *
     * @param PublicRequest $request
     * @param string $tenant
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $tenant, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $tenant = Tenant::findBySignedId($tenant);
        $user = user();
        $tenant = TenantWorkflow::run($transition, $tenant, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($tenant, $user)
    {
        $transitions = [];
        foreach ($tenant->workflow()->transitions($tenant) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.tenant.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'tenant' => $tenant->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $tenant->workflow()->form($value);
            $array['label'] = trans('contact::tenant.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
