<?php

namespace Litecms\Contact\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litecms\Contact\Forms\Vendor as VendorForm;
use Litecms\Contact\Http\Requests\VendorResourceRequest;
use Litecms\Contact\Http\Resources\VendorResource;
use Litecms\Contact\Http\Resources\VendorsCollection;
use Litecms\Contact\Models\Vendor;
use Litecms\Contact\Actions\VendorAction;
use Litecms\Contact\Actions\VendorActions;


/**
 * Resource controller class for vendor.
 */
class VendorResourceController extends BaseController
{

    /**
     * Initialize vendor resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = VendorForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('litecms.contact.modules'), 'contact', guard_url('contact'));
            return $next($request);
        });
    }

    /**
     * Display a list of vendor.
     *
     * @return Response
     */
    public function index(VendorResourceRequest $request)
    {
        $request = $request->all();
        $page = VendorActions::run('paginate', $request);

        $data = new VendorsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('contact::vendor.names'))
            ->view('contact::vendor.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display vendor.
     *
     * @param Request $request
     * @param Model   $vendor
     *
     * @return Response
     */
    public function show(VendorResourceRequest $request, Vendor $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new VendorResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('contact::vendor.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('contact::vendor.show')
            ->output();
    }

    /**
     * Show the form for creating a new vendor.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(VendorResourceRequest $request, Vendor $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new VendorResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('contact::vendor.name'))
            ->view('contact::vendor.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new vendor.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(VendorResourceRequest $request, Vendor $model)
    {
        try {
            $request = $request->all();
            $model = VendorAction::run('store', $model, $request);
            $data = new VendorResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('contact::vendor.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('contact/vendor/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/contact/vendor'))
                ->redirect();
        }

    }

    /**
     * Show vendor for editing.
     *
     * @param Request $request
     * @param Model   $vendor
     *
     * @return Response
     */
    public function edit(VendorResourceRequest $request, Vendor $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new VendorResource($model);
        // return view('contact::vendor.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('contact::vendor.name'))
            ->view('contact::vendor.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the vendor.
     *
     * @param Request $request
     * @param Model   $vendor
     *
     * @return Response
     */
    public function update(VendorResourceRequest $request, Vendor $model)
    {
        try {
            $request = $request->all();
            $model = VendorAction::run('update', $model, $request);
            $data = new VendorResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('contact::vendor.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('contact/vendor/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('contact/vendor/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the vendor.
     *
     * @param Model   $vendor
     *
     * @return Response
     */
    public function destroy(VendorResourceRequest $request, Vendor $model)
    {
        try {

            $request = $request->all();
            $model = VendorAction::run('destroy', $model, $request);
            $data = new VendorResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('contact::vendor.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('contact/vendor/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('contact/vendor/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
